#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Demo script for loading NEMS recordings used in
  Lopez Espejo et al. (2019) PLoS Computational Biology.

This script does three things:
1. load data from a multi-channel recording in primary auditory cortex during
   presentation of vocalization modulated noise (VMN)
2. select data for one stimulus and plot stimulus envelopes with corresponding rasters
   and PSTHs for two cells
3. extract the full stimulus envelope and time-varying spike rate response
   for one cell from the entire experiment.

Requires NEMS toolbox (and dependencies) to load. Pull the master branch from
here:
    https://github.com/LBHB/NEMS/

Created on 2019-09-18
@author: svd
"""

import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
import os

from nems.recording import load_recording
import nems.plots.api as nplt
import nems.epoch as ep
from nems import get_setting

# some settings for matplotlib
font_size = 10
params = {'legend.fontsize': font_size-2,
          'figure.figsize': (8, 6),
          'axes.labelsize': font_size,
          'axes.titlesize': font_size,
          'xtick.labelsize': font_size,
          'ytick.labelsize': font_size,
          'pdf.fonttype': 42,
          'ps.fonttype': 42}
plt.rcParams.update(params)

# ----------------------------------------------------------------------------
# 1. DATA LOADING & PRE-PROCESSING

VMN_data = pd.read_csv('VMN_data.csv', index_col=0)

# pick two cells from same recording
cellid1 = "por074a-a3"
cellid2 = "por074a-d1"

# load the data
file = VMN_data.loc[cellid1, 'file']
uri = "file://VMN/" + file
rec = load_recording(uri)

# get the response signal for the two cells
resp = rec['resp'].rasterize()
resp = resp.extract_channels([cellid1,cellid2])

# get the corresponding stimulus signal
stim = rec['stim']

# get a list of stimulus names
epochs = rec.epochs

# 2. PLOT PSTHs FOR ONE STIMULUS

# find out number of times each stimulus was repeated
epoch_counts = ep.epoch_occurrences(epochs, regex="^STIM_")

# figure out which stimulus/-i were repeated most (ie, validation data)
max_reps = epoch_counts.max()
rep_epochs = epoch_counts.index[epoch_counts == max_reps]

# choose a stimulus, in this case, the high-repeat stimulus:
epoch = rep_epochs[0]

# plot the PSTH for an example stimulus
raster = resp.extract_epoch(epoch)
psth = np.mean(raster, axis=0)
spectrogram = stim._data[epoch]
tt = np.arange(psth.shape[1])/resp.fs

print('Stimulus and response data extracted for stimulus ' + epoch)

# generate plot
fig, axs = plt.subplots(5,1, figsize=(6,8))

nplt.plot_timeseries([tt], [spectrogram.T], ax=axs[0], title=epoch,
                     legend=['Chan 0','Chan 1'], ylabel="Envelope")

[x, y]=np.where(raster[:,0,:].T)
x = x / resp.fs
axs[1].plot(x, y, '.', color='black', markersize=1)
axs[1].set_ylabel('cell {}'.format(cellid1))
axs[1].set_xticklabels([])

axs[2].plot(tt,psth[0,:], color='black', lw=1)
axs[2].set_ylabel('Spikes/sec')
axs[2].set_xticklabels([])

[x, y]=np.where(raster[:,1,:].T)
x = x / resp.fs
axs[3].plot(x, y, '.', color='black', markersize=1)
axs[3].set_ylabel('cell {}'.format(cellid2))
axs[3].set_xticklabels([])

axs[4].plot(tt,psth[1, :], color='black', lw=1)
axs[4].set_ylabel('Spikes/sec')
axs[4].set_xlabel('Time')

# make all x limits the same
axs[0].set_xlim(axs[1].get_xlim())

for ax in axs:
    nplt.ax_remove_box(ax)

plt.tight_layout()

# 3. EXTRACT THE ENTIRE STIMULUS/RESPONSE TIME SERIES FOR ONE CELL

# get big stimulus and response matrices. Note that "time" here is real
# experiment time, so repeats of the test stimulus show up as single-trial
# stimuli.

X = stim.rasterize().as_continuous()
Y = resp.extract_channels([cellid1]).as_continuous()

print('Single-trial stim/resp data extracted to X/Y matrices!')
print('cellid: ', cellid1)
print('X.shape (stimulus): ', X.shape)
print('Y.shape (response): ', Y.shape)

# Bonus. Here's how you get rasters aligned to each stimulus:
# define regex for stimulus epochs
epoch_regex = '^STIM_'
epochs_to_extract = ep.epoch_names_matching(rec.epochs, epoch_regex)
folded_resp = resp.extract_epochs(epochs_to_extract)
